﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using Microsoft.JSInterop;

namespace BlazorAceEditor
{
    public static class AceEditorJsCallbackManager
    {
        private static readonly ConcurrentDictionary<string, Dictionary<string, Delegate>> EventHandlerDict = new();

        public static void AddEventHandler(string elementId, string eventKey, Delegate @delegate)
        {
            var eventHandlerList = EventHandlerDict.GetOrAdd(elementId, (key) => new Dictionary<string, Delegate>());
            eventHandlerList[eventKey] = @delegate;
        }

        public static void DisposeObject(string elementId)
        {
            if (EventHandlerDict.Remove(elementId, out Dictionary<string, Delegate> handlers))
            {
                handlers.Clear();
            }
        }

        [JSInvokable]
        public static object JsCallback(string elementId, string eventKey, object args)
        {
            if (EventHandlerDict.TryGetValue(elementId, out Dictionary<string, Delegate> handlers))
            {
                if (handlers.TryGetValue(eventKey, out Delegate d))
                {
                    var obj = args == null ? d.DynamicInvoke() : d.DynamicInvoke(args);
                    return obj;
                }
            }
            return null;
        }
    }
}